import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class week02_basic_forces extends PApplet {/*

  In this example we look at basic forces acting on a single object. We'll
  use gravity, wind and drag to move our object about the screen.
*/
SimObject so;
boolean showVectors = true;
PFont labelFont;

public void setup() {

	// setup the processing environment
	size(300, 400);
	background(10);
	colorMode(RGB, 255, 255, 255, 255);
	framerate(20);

        // load our font
        labelFont = loadFont("LucidaGrande-32.vlw");
        textFont(labelFont, 14);
                
	// create our SimObject
	Vector2D a = new Vector2D(0.0f,0.0f);
	Vector2D v = new Vector2D(0.0f,0.0f);
	Vector2D l = new Vector2D(150,10);
	so = new SimObject(a, v, l, 1.5f);
	
}

public void draw() {

	// reset our background
	background(10);
	
	// add gravity to our object
	/*
		Note that these are made up values for gravity, for
		the sake of giving our simulation a "feeling" of
		reality.
	*/
	Vector2D gravity = new Vector2D(0, 0.2f);
	so.addForce(gravity);
        drawOverlay(gravity, so.getLocation(), 200, "Gravity");

	// add "wind" to the object
	/*
		Again, we're making up values for this.
	*/
	Vector2D wind = new Vector2D(0.0f, -0.01f);
	so.addForce(wind);
	drawOverlay(wind, so.getLocation(), 200, "Wind");

	// add friction (drag) to our object
	/*
	
		We'll define an area of the screen that has "friction"
		that slows down our object as it moves through.
		
		Drag is calculated as being an opposing force to the
		velocity of the object, so we'll use the value of
		our object's velocity to calculate the drag force.
	*/
	float drag_start = 200.0f;
	float drag_height = 100.0f;
	
	// determine if we're inside of the drag area
	if ( (so.getLocation().getY() > drag_start) && (so.getLocation().getY() < drag_start + drag_height) ) {
	
		// set our coefficient of friction
		float c = -0.03f;
		
		// get our current velocity
		Vector2D objectVelocity = so.getVelocity();
		
		// determine the drag force
		Vector2D drag = objectVelocity.multiply(c);
		
		// add the drag force
		so.addForce(drag);

                drawOverlay(drag, so.getLocation(), 200, "Friction");
                		
	}

	// set our simulation in motion
	so.simulate();
	
	// draw the drag area
	noStroke();
	fill(128, 16, 16, 225);
	rectMode(CORNER);
	rect(0, drag_start, width, drag_height);
	
}

/*
	This is our utility function for drawing overlays
	describing the force vectors acting on our object.
*/
public void drawOverlay(Vector2D v, Vector2D location, float scale, String name) {

  if (showVectors == true) {
	// save our drawing environment
	pushMatrix();
	
	// move to our position to draw our force vector
	translate(location.getX(), location.getY());

	// drawing settings
	stroke(255);
	float arrowSize = 10;
	float arrowLength = v.magnitude() * scale;
	
	// rotate so that the direction of our drawing is the heading
	// of our vector
	rotate(v.heading());
	
	// draw the arrow
	line(0, 0, arrowLength, 0);
	line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2);
	line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2 * -1);
	
	// label our arrow
        fill(255, 255, 255, 255);
	text(name, arrowLength, 0);

	// reset our drawing environment
	popMatrix();
  }
}

public void mousePressed() {
	showVectors = !showVectors;
}

/*

	This is an object in our simulation environment, with a location,
	velocity and acceleration. Our object also has a mass, which effects
	the forces acting upon the object.
	
*/

public class SimObject {

	Vector2D loc;
	Vector2D vel;
	Vector2D accel;
	
	float mass;
	float max_velocity;
	float bounciness;
	float objectSize;
	
	public SimObject(Vector2D loc_, Vector2D vel_, Vector2D accel_, float mass_) {
	
		// Save the properties we've been constructed with
		loc = loc_;
		vel = vel_;
		accel = accel_;
		mass = mass_;
		
		// set some basic defaults for our other properties
		max_velocity = 20.0f;
		bounciness = 0.98f;
		objectSize = 20.0f;
		
	}
	
	public Vector2D getLocation() {
		return loc;
	}
	
	public void setLocation(Vector2D v) {
		loc = v;
	}
	
	public Vector2D getVelocity() {
		return vel;
	}
	
	public void setVelocity(Vector2D v) {
		vel = v;
	}
	
	public Vector2D getAcceleration() {
		return accel;
	}
	
	public void setAcceleration(Vector2D v) {
		accel = v;
	}
	
	public float getMass() {
		return mass;
	}
	
	public void setMass(float m_) {
		mass = m_;
	}
	
	public void addForce(Vector2D v) {
	
		// scale the force by our mass
		v = v.divide(mass);
		
		// add the force to our acceleration
		accel = accel.add(v);
		
	}

	public void simulate() {
		
		// Simulate a step in time for our object
		update();
		contain();
		display();
		
	}
	
	public void update() {
	
		// update our values for velocity and
		// location to reflect our movement
		
		// add acceleration to velocity
		vel = vel.add(accel);
		
		// limit our velocity to a maximum speed
		vel.limit(max_velocity);
		
		// move our object according to our velocity
		loc = loc.add(vel);
		
		// reset the accerlation forces to zero in
		// preperation for the next step in time
		accel.setXY(0.0f, 0.0f);
	}
	
	public void contain() {
	
		// We want to contain our object to the confines
		// of the screen, so here we check to make
		// sure we're within the screen bounds. If we're
		// at the edge of the screen, we "bounce" back
		
		// Contain vertical movement
		if ( (loc.getY() > height - objectSize / 2) || (loc.getY() < objectSize / 2) ) {
		
                        // flip our velocity
                        vel.setY(bounciness * -1.0f * vel.getY());
                        
                        // move our position
                        if (loc.getY() > (height / 2) ) {
                          loc.setY(height - objectSize / 2);
			} else {
                          loc.setY(objectSize / 2);
                        }
			
		}

		// Contain horizontal movement
		if ( (loc.getX() > width - objectSize / 2) || (loc.getX() < objectSize / 2) ) {
		
			vel.setX(bounciness * -1.0f * vel.getX());
			
		}
	}
	
	public void display() {
	
		// display our object on screen
		rectMode(CENTER);
		noStroke();
		fill(32, 64, 128, 200);
		rect(loc.getX(), loc.getY(), objectSize, objectSize);
		
	}
}
}