import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class week02_attractor extends PApplet {/*
  
  In this example we have a single object moving around a single attractor object.
  Think of this like a planet orbiting the sun.
  
*/

// Our simulation object
SimObject so;

// Our object exerting gravitational pull
Attractor att;

// Flag for showing the force vectors
boolean showVectors = true;

// Font for labeling vectors
PFont labelFont;

public void setup() {
 
    size(400, 400);
    background(15);
    
    // setup our sim object
    Vector2D a = new Vector2D(0.0f, 0.0f);
    Vector2D v = new Vector2D(0.0f, -0.2f);
    Vector2D l = new Vector2D(350, 350);
    so = new SimObject(l, v, a, 10);
    //so.setContained(true);
    
    // setup the attractor
    att = new Attractor(new Vector2D(width / 2, height / 2), 10, 0.2f);

    // load our font
    labelFont = loadFont("LucidaGrande-32.vlw");
    textFont(labelFont, 16); 
}

public void draw() {

    background(15);
    
    // figure out the force exerted on our SimObject by the attractor
    Vector2D attForce = att.getGravityForce(so);
    
    // add the forces to our object
    so.addForce(attForce);
    drawOverlay(so.getVelocity(), so.getLocation(), 50, "Velocity");
    drawOverlay(so.getAcceleration(), so.getLocation(), 5000, "Acceleration");
  
   // println("Velocity: " + so.getVelocity().getX() + ", " + so.getVelocity().getY());  
    // let the objects run
    so.simulate();
    att.simulate();
    
}

public void mousePressed() {
    att.clicked(mouseX, mouseY);
}

public void mouseReleased() {
    att.stopDragging();
}

public void keyPressed() {
    showVectors = !showVectors;
}

/*

	An attractor represents an object with gravitational attraction
	for the other objects around it. The attractor is stationary,
	drawing other objects toward it. We'll use the formulas
	for the gravitational attraction between two bodies to
	determine how this Attractor effects other objects.
*/

public class Attractor {

	// mass of our attractor
	float mass;
	
	// The Gravitational constant
	float G;
	
	// location of our attractor
	Vector2D location;
	
	// Variables for moving our attractor around the screen
	boolean isDragging = false;
	Vector2D dragOffset;
	
	public Attractor(Vector2D loc_, float mass_, float g_) {
	
		// save our construction properties
		location = loc_;
		mass = mass_;
		G = g_;
		
		// give our other properties default values
		dragOffset = new Vector2D(0.0f, 0.0f);
	}
	
	public void simulate() {
	
		display();
		drag();
		
	}
	
	public Vector2D getGravityForce(SimObject so) {
	
		// get the direction of the gravitational force
		Vector2D forceDirection = Vector2D.subtract(location, so.getLocation());
		
		// get the distance between the objects
		float distance = forceDirection.magnitude();
		
		// calculate the magnitude of the gravitational force
		forceDirection.normalize();
		float force = (G * mass * so.getMass()) / (distance * distance);
		forceDirection = forceDirection.multiply(force);
		
		return forceDirection;
		
	}
	
	public void display() {
	
		// display our attractor
		rectMode(CENTER);
		noStroke();
		
		if (isDragging) {
			fill(128, 128, 128);
		} else {
			fill(16, 128, 16);
		}
		
		ellipse(location.getX(), location.getY(), mass * 2, mass * 2);
		
	}

	public void clicked(int x, int y) {
	
		float distance = dist(x, y, location.getX(), location.getY());
		if (distance < mass) {
			isDragging = true;
			dragOffset.setX(location.getX() - x);
			dragOffset.setY(location.getY() - y);
		}
	}

	public void stopDragging() {
		isDragging = false;
	}
	
	public void drag() {
		if (isDragging) {
			location.setX(mouseX + dragOffset.getX());
			location.setY(mouseY + dragOffset.getY());
		}
	}

}

/*

	This is an object in our simulation environment, with a location,
	velocity and acceleration. Our object also has a mass, which effects
	the forces acting upon the object.
	
*/

public class SimObject {

	Vector2D loc;
	Vector2D vel;
	Vector2D accel;
	
	float mass;
	float max_velocity;
	float bounciness;
	float objectSize;
	
        boolean contained;
        
	public SimObject(Vector2D loc_, Vector2D vel_, Vector2D accel_, float mass_) {
	
		// Save the properties we've been constructed with
		loc = loc_;
		vel = vel_;
		accel = accel_;
		mass = mass_;
		
		// set some basic defaults for our other properties
		max_velocity = 5.0f;
		bounciness = 0.98f;
		objectSize = 20.0f;
	  
                contained = false;	
	}
	
        public boolean isContained() {
                return contained;
        }
        
        public void setContained(boolean c) {
                contained = c;
        }
        
	public Vector2D getLocation() {
		return loc;
	}
	
	public void setLocation(Vector2D v) {
		loc = v;
	}
	
	public Vector2D getVelocity() {
		return vel;
	}
	
	public void setVelocity(Vector2D v) {
		vel = v;
	}
	
	public Vector2D getAcceleration() {
		return accel;
	}
	
	public void setAcceleration(Vector2D v) {
		accel = v;
	}
	
	public float getMass() {
		return mass;
	}
	
	public void setMass(float m_) {
		mass = m_;
	}
	
	public void addForce(Vector2D v) {
	
		// scale the force by our mass
//		v = v.divide(mass);
		
		// add the force to our acceleration
		accel = accel.add(v);
		
	}

	public void simulate() {
		
		// Simulate a step in time for our object
		update();

                if (contained) {
      		        contain();
                }
		display();
		
	}
	
	public void update() {
	
		// update our values for velocity and
		// location to reflect our movement
		
		// add acceleration to velocity
		vel = vel.add(accel);
		
		// limit our velocity to a maximum speed
		vel.limit(max_velocity);
		
		// move our object according to our velocity
		loc = loc.add(vel);
		
		// reset the accerlation forces to zero in
		// preperation for the next step in time
		accel.setXY(0.0f, 0.0f);
	}
	
	public void contain() {
	
		// We want to contain our object to the confines
		// of the screen, so here we check to make
		// sure we're within the screen bounds. If we're
		// at the edge of the screen, we "bounce" back
		
		// Contain vertical movement
		if ( (loc.getY() > height - objectSize / 2) || (loc.getY() < objectSize / 2) ) {
		
                        // flip our velocity
                        vel.setY(bounciness * -1.0f * vel.getY());
                        
                        // move our position
                        if (loc.getY() > (height / 2) ) {
                          loc.setY(height - objectSize / 2);
			} else {
                          loc.setY(objectSize / 2);
                        }
			
		}

		// Contain horizontal movement
		if ( (loc.getX() > width - objectSize / 2) || (loc.getX() < objectSize / 2) ) {
		
			vel.setX(bounciness * -1.0f * vel.getX());
			
		}
	}
	
	public void display() {
	
		// display our object on screen
		rectMode(CENTER);
		noStroke();
		fill(32, 64, 128, 200);
		rect(loc.getX(), loc.getY(), objectSize, objectSize);
		
	}
}

/*
	This is our utility function for drawing overlays
	describing the force vectors acting on our object.
*/
public void drawOverlay(Vector2D v, Vector2D location, float tscale, String name) {

  if (showVectors == true) {
	// save our drawing environment
	pushMatrix();
	
	// move to our position to draw our force vector
	translate(location.getX(), location.getY());

	// drawing settings
	stroke(255);
	float arrowSize = 10;
	float arrowLength = v.magnitude() * tscale;
	
	// rotate so that the direction of our drawing is the heading
	// of our vector
	rotate(v.heading());
	
	// draw the arrow
	line(0, 0, arrowLength, 0);
	line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2);
	line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2 * -1);
	
	// label our arrow
        fill(255, 255, 255, 255);
	text(name, arrowLength, 0);

	// reset our drawing environment
	popMatrix();
  }
}
}