import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class week02_mutual_attraction extends PApplet {/*

  In this example each SimObject exerts force on every other SimObject; we've
  effectively combined the Attractor and SimObject from the Attractor_Bodies
  example.
*/

// set our maximum number of objects
public static final int MAX_BODIES = 10;

// declare our array of sim objects
SimObject[] so = new SimObject[MAX_BODIES];

// variables for displaying our labels
int showVectors = 1;
PFont labelFont;

public void setup() {

    // setup our environment
    size(500, 500);
    background(15);
    smooth();
    
    // load our font
    labelFont = loadFont("LucidaGrande-16.vlw");
    textFont(labelFont, 16);
    
    // setup our objects
    reset();
}

public void draw() {

    // clear the screen
    background(15);
    
    /*
        To create a situation where each object exerts a force on 
        every other object, we must loop through all of the objects
        twice; with the first loop we choose which object we are
        calculating hte force for, in the inner loop we are calculating
        the force of every object on the object we chose.
    */
    for (int i = 0; i < so.length; i++) {
        for (int j = 0; j < so.length; j++) {
            
            // an object can't exert force on itself
            if (i != j) {
            
                // calculate the gravitational pull between the two
                Vector2D f = so[i].getGravityForce(so[j]);
                so[i].addForce(f);
                
            }
        }
        
        // draw our overlay
        drawOverlay(so[i].getVelocity(), so[i].getLocation(), 50, "Velocity");
        drawOverlay(so[i].getAcceleration(), so[i].getLocation(), 1000, "Acceleration");
        
        // let the object simulate
        so[i].simulate();
        
    }
    
}

public void reset() {

    // setup each of our objects
    for (int i = 0; i < so.length; i++) {
    
        Vector2D a = new Vector2D(0.0f, 0.0f);
        Vector2D v = new Vector2D(0.0f, 0.0f);
        Vector2D l = new Vector2D(random(width), random(height));
        so[i] = new SimObject(l, v, a, random(5, 15), 0.2f);
        
        so[i].setContained(true);
    }
}

public void mousePressed() {
    reset();
}

public void keyPressed() {
    showVectors++;
    if (showVectors > 2) {
        showVectors = 0;
    }
}


/*

	This is an object in our simulation environment, with a location,
	velocity and acceleration. Our object also has a mass, which effects
	the forces acting upon the object.

        For this version of our SimObject code we need to include the properties
        and methods for calculating gravitational attraction.	
*/

public class SimObject {

	Vector2D loc;
	Vector2D vel;
	Vector2D accel;
	
	float mass;
	float max_velocity;
	float bounciness;
	float objectSize;
	float G;

        boolean contained;
        
	public SimObject(Vector2D loc_, Vector2D vel_, Vector2D accel_, float mass_, float g_) {
	
		// Save the properties we've been constructed with
		loc = loc_;
		vel = vel_;
		accel = accel_;
		mass = mass_;
		G = g_;

		// set some basic defaults for our other properties
		max_velocity = 5.0f;
		bounciness = 0.98f;
		objectSize = mass * 5.0f;
	  
                contained = false;	
	}
	
        public boolean isContained() {
                return contained;
        }
        
        public void setContained(boolean c) {
                contained = c;
        }
        
	public Vector2D getLocation() {
		return loc;
	}
	
	public void setLocation(Vector2D v) {
		loc = v;
	}
	
	public Vector2D getVelocity() {
		return vel;
	}
	
	public void setVelocity(Vector2D v) {
		vel = v;
	}
	
	public Vector2D getAcceleration() {
		return accel;
	}
	
	public void setAcceleration(Vector2D v) {
		accel = v;
	}
	
	public float getMass() {
		return mass;
	}
	
	public void setMass(float m_) {
		mass = m_;
	}
	
	public void addForce(Vector2D v) {
	
		// scale the force by our mass
//		v = v.divide(mass);
		
		// add the force to our acceleration
		accel = accel.add(v);
		
	}

	public void simulate() {
		
		// Simulate a step in time for our object
		update();

                if (contained) {
      		        contain();
                }
		display();
		
	}
	
	public void update() {
	
		// update our values for velocity and
		// location to reflect our movement
		
		// add acceleration to velocity
		vel = vel.add(accel);
		
		// limit our velocity to a maximum speed
		vel.limit(max_velocity);
		
		// move our object according to our velocity
		loc = loc.add(vel);
		
		// reset the accerlation forces to zero in
		// preperation for the next step in time
		accel.setXY(0.0f, 0.0f);
	}
	
	public void contain() {
	
		// We want to contain our object to the confines
		// of the screen, so here we check to make
		// sure we're within the screen bounds. If we're
		// at the edge of the screen, we "bounce" back
		
		// Contain vertical movement
		if ( (loc.getY() > height - objectSize / 2) || (loc.getY() < objectSize / 2) ) {
		
                        // flip our velocity
                        vel.setY(bounciness * -1.0f * vel.getY());
                        
                        // move our position
                        if (loc.getY() > (height / 2) ) {
                          loc.setY(height - objectSize / 2);
			} else {
                          loc.setY(objectSize / 2);
                        }
			
		}

		// Contain horizontal movement
		if ( (loc.getX() > width - objectSize / 2) || (loc.getX() < objectSize / 2) ) {
		
			vel.setX(bounciness * -1.0f * vel.getX());
			
		}
	}
	
	public void display() {
	
		// display our object on screen
		rectMode(CENTER);
		noStroke();
		fill(32, 64, 128, 200);
		rect(loc.getX(), loc.getY(), objectSize, objectSize);
		
	}

        public Vector2D getGravityForce(SimObject so) {
        
            Vector2D direction = Vector2D.subtract(so.getLocation(), loc);
            float distance = direction.magnitude();
            
            // keep a limit on the distance used in the calculation to
            // limit extreme results
            distance = constrain(distance, 20.0f, 100.0f);
            direction.normalize();
            float force = (G * mass * so.getMass()) / (distance * distance);
            direction = direction.multiply(force);
            return direction;
        }
}

/*
	This is our utility function for drawing overlays
	describing the force vectors acting on our object.
*/
public void drawOverlay(Vector2D v, Vector2D location, float tscale, String name) {

  if (showVectors > 0) {
	// save our drawing environment
	pushMatrix();
	
	// move to our position to draw our force vector
	translate(location.getX(), location.getY());

	// drawing settings
	stroke(255);
	float arrowSize = 10;
	float arrowLength = v.magnitude() * tscale;
	
	// rotate so that the direction of our drawing is the heading
	// of our vector
	rotate(v.heading());
	
	// draw the arrow
	line(0, 0, arrowLength, 0);
	//line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2);
	//line(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2 * -1);
        noStroke();
        fill(255, 255, 255, 255);
	triangle(arrowLength, 0, arrowLength - arrowSize, arrowSize / 2, arrowLength - arrowSize, arrowSize / 2 * -1);

        if (showVectors == 2) {
            
            // label our arrow
            fill(255, 255, 255, 255);
	    text(name, arrowLength, 0);
        }

	// reset our drawing environment
	popMatrix();
  }
}
}